/*
 * catberry
 *
 * Copyright (c) 2015 Denis Rechkunov and project contributors.
 *
 * catberry's license follows:
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * This license applies to all parts of catberry that are not externally
 * maintained libraries.
 */

'use strict';

module.exports = StoreLoader;

var URI = require('catberry-uri').URI,
	moduleHelper = require('../helpers/moduleHelper'),
	requireHelper = require('../helpers/requireHelper'),
	util = require('util'),
	path = require('path'),
	LoaderBase = require('../base/LoaderBase');

var ERROR_CONSTRUCTOR_SHOULD_BE_FUNCTION =
	'Store file %s should export a constructor function',
	INFO_WATCHING_FILES = 'Watching stores for changes',
	INFO_STORE_CHANGED = 'Store "%s" has been changed, reinitializing...',
	INFO_STORE_ADDED = 'Store "%s" has been added, initializing...',
	INFO_STORE_UNLINKED = 'Store "%s" has been unlinked, removing...';

util.inherits(StoreLoader, LoaderBase);

/**
 * Creates new instance of the store loader.
 * @param {ServiceLocator} $serviceLocator Locator to resolve dependencies.
 * @param {boolean} isRelease Release mode flag.
 * @constructor
 * @extends LoaderBase
 */
function StoreLoader($serviceLocator, isRelease) {
	this._logger = $serviceLocator.resolve('logger');
	this._eventBus = $serviceLocator.resolve('eventBus');
	this._storeFinder = $serviceLocator.resolve('storeFinder');
	this._isRelease = Boolean(isRelease);
	LoaderBase.call(this, $serviceLocator.resolveAll('storeTransform'));
}

/**
 * Current release flag.
 * @type {boolean}
 * @private
 */
StoreLoader.prototype._isRelease = false;

/**
 * Current map of loaded stores by names.
 * @type {Object}
 * @private
 */
StoreLoader.prototype._loadedStores = null;

/**
 * Current logger.
 * @type {Logger}
 * @private
 */
StoreLoader.prototype._logger = null;

/**
 * Current event bus.
 * @type {EventEmitter}
 * @private
 */
StoreLoader.prototype._eventBus = null;

/**
 * Current store finder.
 * @type {StoreFinder}
 * @private
 */
StoreLoader.prototype._storeFinder = null;

/**
 * Loads all stores into a memory.
 * @returns {Promise<Object>} Promise for map of loaded stores.
 */
StoreLoader.prototype.load = function () {
	if (this._loadedStores) {
		return Promise.resolve(this._loadedStores);
	}

	var self = this,
		result = Object.create(null);

	return this._storeFinder.find()
		.then(function (details) {
			var storePromises = Object.keys(details)
				.map(function (storeName) {
					return self._getStore(details[storeName]);
				});

			return Promise.all(storePromises);
		})
		.then(function (stores) {
			stores.forEach(function (store) {
				if (!store || typeof (store) !== 'object') {
					return;
				}

				result[store.name] = store;
			});

			self._loadedStores = result;

			if (!self._isRelease) {
				self._logger.info(INFO_WATCHING_FILES);
				self._storeFinder.watch();
				self._handleChanges();
			}

			self._eventBus.emit('allStoresLoaded', result);
			return self._loadedStores;
		});
};

/**
 * Gets current map of stores by names.
 * @returns {Object} Map of stores by names.
 */
StoreLoader.prototype.getStoresByNames = function () {
	return this._loadedStores || Object.create(null);
};

/**
 * Gets store object by found store details.
 * @param {Object} storeDetails Found details.
 * @returns {Object} Store object.
 * @private
 */
StoreLoader.prototype._getStore = function (storeDetails) {
	var constructor;
	try {
		constructor = require(
			requireHelper.getAbsoluteRequirePath(storeDetails.path)
		);
	} catch (e) {
		this._eventBus.emit('error', e);
		return Promise.resolve(null);
	}

	if (typeof (constructor) !== 'function') {
		var errorMessage = util.format(
			ERROR_CONSTRUCTOR_SHOULD_BE_FUNCTION,
			storeDetails.path
		);
		this._eventBus.emit('error', new Error(errorMessage));
		return Promise.resolve(null);
	}

	var result = Object.create(storeDetails);
	result.constructor = constructor;

	var self = this;
	return this._applyTransforms(result)
		.then(function (transformed) {
			self._eventBus.emit('storeLoaded', transformed);
			return transformed;
		})
		.catch(function (error) {
			self._eventBus.emit('error', error);
			return null;
		});
};

/**
 * Handles watch changes.
 * @private
 */
StoreLoader.prototype._handleChanges = function () {
	var self = this,
		loadStore = function (storeDetails) {
			self._getStore(storeDetails)
				.then(function (store) {
					self._loadedStores[storeDetails.name] = store;
				});
		};

	this._storeFinder
		.on('add', function (storeDetails) {
			self._logger.info(util.format(
				INFO_STORE_ADDED, storeDetails.path
			));
			requireHelper.clearCacheKey(
				requireHelper.getAbsoluteRequirePath(storeDetails.path)
			);

			loadStore(storeDetails);
		})
		.on('change', function (storeDetails) {
			self._logger.info(util.format(
				INFO_STORE_CHANGED, storeDetails.path
			));
			requireHelper.clearCacheKey(
				requireHelper.getAbsoluteRequirePath(storeDetails.path)
			);
			loadStore(storeDetails);
		})
		.on('unlink', function (storeDetails) {
			self._logger.info(util.format(
				INFO_STORE_UNLINKED, storeDetails.path
			));
			requireHelper.clearCacheKey(
				requireHelper.getAbsoluteRequirePath(storeDetails.path)
			);
			delete self._loadedStores[storeDetails.name];
		});
};